@@ -648,9 +648,10 @@ def counter(tokens, default_integer):
 @property('margin-right')
 @property('margin-bottom')
 @property('margin-left')
+@property('text-underline-offset')
 @single_token
 def lenght_precentage_or_auto(token):
-    """``margin-*`` properties validation."""
+    """``margin-*`` and various other properties validation."""
     length = get_length(token, percentage=True)
     if length:
         return length
@@ -888,7 +889,7 @@ def font_feature_settings(tokens):
 @single_keyword
 def font_variant_alternates(keyword):
     # TODO: support other values
-    # See https://www.w3.org/TR/css-fonts-3/#font-variant-caps-prop
+    # See https://drafts.csswg.org/css-fonts/#font-variant-alternates-prop
     return keyword in ('normal', 'historical-forms')
 
 
@@ -1009,6 +1010,15 @@ def spacing(token):
         return length
 
 
+@property()
+@single_token
+def outline_offset(token):
+    """Validation for ``outline-offset``."""
+    length = get_length(token)
+    if length:
+        return length
+
+
 @property()
 @single_token
 def line_height(token):
@@ -1227,6 +1237,17 @@ def text_decoration_style(keyword):
         return keyword
 
 
+@property()
+@single_token
+def text_decoration_thickness(token):
+    """``text-decoration-thickness`` property validation."""
+    length = get_length(token, percentage=True)
+    if length:
+        return length
+    if keyword := get_keyword(token) in ('auto', 'from-font'):
+        return keyword
+
+
 @property()
 @single_token
 def text_indent(token):
